@@ -69,12 +69,21 @@
       (set-buffer (car buffers))
       (let ((inhibit-read-only t)
 	    buffer-read-only
-	    (start (goto-char (point-max))))
+	    (start (goto-char (point-max)))
+	    (buffer (current-buffer))
+	    window
+	    point)
 	(insert (format-time-string "%H:%M") " " string)
+	(setq point (point))
 	(if (and (not (riece-frozen (current-buffer)))
-		 (get-buffer-window (current-buffer)))
-	    (set-window-point (get-buffer-window (current-buffer))
-			      (point)))
+		 (setq window (get-buffer-window (current-buffer)))
+		 (not (pos-visible-in-window-p point window)))
+	    (save-excursion		;save-selected-window changes
+					;current buffer
+	      (save-selected-window
+		(select-window window)
+		(goto-char point)	;select-window changes current point
+		(recenter riece-window-center-line))))
 	(run-hook-with-args 'riece-after-insert-functions start (point))))
     (setq buffers (cdr buffers))))
 
